#include <stdio.h>
#include <malloc.h>

/*****************************************************************************
 *	Display the error message and exit to shell.
 *****************************************************************************/
void errmsg(error_message)
char error_message[] ;
   {
   void exit() ;
   
   fprintf(stderr, "Program encounters run-time error...\n") ;
   fprintf(stderr, "%s\n", error_message) ;
   fprintf(stderr, "...now exiting to system...\n") ;
   exit(1) ;
   }


/*****************************************************************************
 *	Allocate memory for integer matrix.
 *****************************************************************************/
int **imatrix(row, col)
int row, col ;
   {
   int		i, **m ;

   m = (int **) malloc((unsigned) row * sizeof(int*)) ;
   if (!m) errmsg("Memory allocation failure") ;
   for (i = 0 ; i < row ; i++)
      {
      m[i] = (int *) malloc((unsigned) col * sizeof(int)) ;
      if (!m[i]) errmsg("Memory alloation failure") ;
      }
   return m ;
   }


/*****************************************************************************
 *	Free the allocated memory of integer matrix.
 *****************************************************************************/
void free_imatrix(m, row, col)
int **m ;
int row, col ;
   {
   int		i ;
   
   for (i = row - 1 ; i >= 0 ; i--)
      free((char*) m[i]) ;
   free((char*) m) ;
   }


/*****************************************************************************
 *	Allocate memory for short matrix.
 *****************************************************************************/
short **smatrix(row, col)
int row, col ;
   {
   short	**m ;
   int		i ;

   m = (short **) malloc((unsigned) row * sizeof(short*)) ;
   if (!m) errmsg("Memory allocation failure") ;
   for (i = 0 ; i < row ; i++)
      {
      m[i] = (short *) malloc((unsigned) col * sizeof(short)) ;
      if (!m[i]) errmsg("Memory alloation failure") ;
      }
   return m ;
   }


/*****************************************************************************
 *	Free the allocated memory of integer matrix.
 *****************************************************************************/
void free_smatrix(m, row, col)
short **m ;
int row, col ;
   {
   int		i ;
   
   for (i = row - 1 ; i >= 0 ; i--)
      free((char*) m[i]) ;
   free((char*) m) ;
   }


/*****************************************************************************
 *	Allocate memory for char matrix.
 *****************************************************************************/
char **cmatrix(row, col)
int row, col ;
   {
   int		i ;
   char		**m ;

   m = (char **) malloc((unsigned) row * sizeof(char*)) ;
   if (!m) errmsg("Memory allocation failure") ;
   for (i = 0 ; i < row ; i++)
      {
      m[i] = (char *) malloc((unsigned) col * sizeof(char)) ;
      if (!m[i]) errmsg("Memory alloation failure") ;
      }
   return m ;
   }


/*****************************************************************************
 *	Free the allocated memory of char matrix.
 *****************************************************************************/
void free_cmatrix(m, row, col)
char **m ;
int row, col ;
   {
   int		i ;
   
   for (i = row - 1 ; i >= 0 ; i--)
      free((char*) m[i]) ;
   free((char*) m) ;
   }


/*****************************************************************************
 *	Allocate memory for unsigned char matrix.
 *****************************************************************************/
unsigned char **ucmatrix(row, col)
int row, col ;
   {
   unsigned char	**m ;
   int			i ;

   m = (unsigned char **) malloc((unsigned) row * sizeof(unsigned char*)) ;
   if (!m) errmsg("Memory allocation failure") ;
   for (i = 0 ; i < row ; i++)
      {
      m[i] = (unsigned char *) malloc((unsigned) col * sizeof(unsigned char)) ;
      if (!m[i]) errmsg("Memory alloation failure") ;
      }
   return m ;
   }


/*****************************************************************************
 *	Free the allocated memory of integer matrix.
 *****************************************************************************/
void free_ucmatrix(m, row, col)
unsigned char **m ;
int row, col ;
   {
   int		i ;
   
   for (i = row - 1 ; i >= 0 ; i--)
      free((char*) m[i]) ;
   free((char*) m) ;
   }


/*****************************************************************************
 *	Allocate memory for floating point number matrix.
 *****************************************************************************/
float **fmatrix(row, col)
int row, col ;
   {
   int		i ;
   float	**m ;

   m = (float **) malloc((unsigned) row * sizeof(float*)) ;
   if (!m) errmsg("Memory allocation failure") ;
   for (i = 0 ; i < row ; i++)
      {
      m[i] = (float *) malloc((unsigned) col * sizeof(float)) ;
      if (!m[i]) errmsg("Memory alloation failure") ;
      }
   return m ;
   }


/*****************************************************************************
 *	Free the allocated memory of floating point number matrix.
 *****************************************************************************/
void free_fmatrix(m, row, col)
float **m ;
int row, col ;
   {
   int		i ;
   
   for (i = row - 1 ; i >= 0 ; i--)
      free((char*) m[i]) ;
   free((char*) m) ;
   }


/*****************************************************************************
 *	Allocate memory for integer vector.
 *****************************************************************************/
int *ivector(dim)
int dim ;
   {
   int		i, *v ;

   v = (int *) malloc((unsigned) dim * sizeof(int)) ;
   if (!v) errmsg("Memory allocation failure") ;
   return v ;
   }


/*****************************************************************************
 *	Free the allocated memory of integer vector.
 *****************************************************************************/
void free_ivector(v, dim)
int *v ;
int dim ;
   {
   free((char*) v) ;
   }


/*****************************************************************************
 *	Allocate memory for character vector.
 *****************************************************************************/
char *cvector(dim)
int dim ;
   {
   int		i ;
   char		*v ;

   v = (char *) malloc((unsigned) dim * sizeof(char)) ;
   if (!v) errmsg("Memory allocation failure") ;
   return v ;
   }


/*****************************************************************************
 *	Free the allocated memory of character vector.
 *****************************************************************************/
void free_cvector(v, dim)
char *v ;
int dim ;
   {
   free((char*) v) ;
   }


/*****************************************************************************
 *	Allocate memory for floating point number vector.
 *****************************************************************************/
float *fvector(dim)
int dim ;
   {
   int		i ;
   float	*v ;

   v = (float *) malloc((unsigned) dim * sizeof(float)) ;
   if (!v) errmsg("Memory allocation failure") ;
   return v ;
   }


/*****************************************************************************
 *	Free the allocated memory of floating point number  vector.
 *****************************************************************************/
void free_fvector(v, dim)
float *v ;
int dim ;
   {
   free((char*) v) ;
   }


/*****************************************************************************
 *      Allocate memory for double precision floating point number matrix.
 *****************************************************************************/
double **dmatrix(row, col)
int row, col ;
   {
   int          i ;
   double       **m ;
           
   m = (double **) malloc((unsigned) row * sizeof(double*)) ;
   if (!m) errmsg("Memory allocation failure") ;
   for (i = 0 ; i < row ; i++)
      {
      m[i] = (double *) malloc((unsigned) col * sizeof(double)) ;
      if (!m[i]) errmsg("Memory alloation failure") ;
      }
   return m ;
   }
                                                  
                                                  
/*****************************************************************************
 *      Free the allocated memory of floating point number matrix.
 *****************************************************************************/
void free_dmatrix(m, row, col)
double **m ;
int row, col ;
   {
   int          i ;
                                                          
   for (i = row - 1 ; i >= 0 ; i--)
      free((char*) m[i]) ;
   free((char*) m) ;
   }


/*****************************************************************************
 *      Allocate memory for floating point vector.
 *****************************************************************************/
double *dvector(dim)
int dim ;
   {
   int          i ;
   double       *v ;
           
   v = (double *) malloc((unsigned) dim * sizeof(double)) ;
   if (!v) errmsg("Memory allocation failure") ;
   return v ;
   }
                       
                       
/*****************************************************************************
 *      Free the allocated memory of floating point vector.
 *****************************************************************************/
void free_dvector(v, dim)
double *v ;
int dim ;
   {
   free((char*) v) ;
   }


/*****************************************************************************
 *      Allocate memory for short vector.
 *****************************************************************************/
short *svector(dim)
int dim ;
   {
   int          i ;
   short       *v ;
           
   v = (short *) malloc((unsigned) dim * sizeof(short)) ;
   if (!v) errmsg("Memory allocation failure") ;
   return v ;
   }
                       
                       
/*****************************************************************************
 *      Free the allocated memory of short vector.
 *****************************************************************************/
void free_svector(v, dim)
short *v ;
int dim ;
   {
   free((char*) v) ;
   }
                                                                                                        